package com.example.timer.timerservice;

import ch.qos.logback.core.util.TimeUtil;
import com.example.timer.Helloworldjob;
import com.example.timer.info.Timerinfo;
import com.example.timer.util.timeutil;
import jakarta.annotation.PostConstruct;
import jakarta.annotation.PreDestroy;
import org.apache.commons.logging.Log;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Service

public class SchedulerService {
    private static final Logger log = LoggerFactory.getLogger(SchedulerService.class);
    private final Scheduler scheduler;

    @Autowired
    public SchedulerService(Scheduler scheduler) {
        this.scheduler=scheduler;
    }

    public void schedule(final Class jobclass, final Timerinfo info) {
        final JobDetail jobDetail = timeutil.buildJobDetail(jobclass,info);
        final Trigger trigger = timeutil.buildTrigger(jobclass,info);

        try{
            scheduler.scheduleJob(jobDetail,trigger);
        } catch (SchedulerException e) {
            log.error(e.getMessage(),e);
        }


        }
        public void updateTimer(final String timerId, final Timerinfo info ) {
        try {
            final JobDetail jobDetail = scheduler.getJobDetail(new JobKey(timerId));
            if (jobDetail ==null) {
                log.error("Failed to find to timer with ID '{}", timerId);
                return;
            }
            jobDetail.getJobDataMap().put(timerId,info);
        } catch ( final SchedulerException e) {
            log.error(e.getMessage(),e);
        }


        }
        public boolean deleteTimer(final String timerId) {
        try{
            return scheduler.deleteJob(new JobKey(timerId));
        } catch (SchedulerException e) {
            log.error(e.getMessage(),e);
            return false;
        }
        }

    @PostConstruct
    public void init() {
        try{
            scheduler.start();
            scheduler.getListenerManager().addTriggerListener(new Simpletrigger(this));
        } catch (SchedulerException e) {
            log.error(e.getMessage(),e);
        }
    }
    public List<Timerinfo> getAllRunningTimers() {
        try {
            scheduler.getJobKeys(GroupMatcher.anyGroup())
                    .stream()
                    .map(jobKey -> {
                        try {
                            final JobDetail jobDetail = scheduler.getJobDetail(jobKey);
                            return (Timerinfo) jobDetail.getJobDataMap().get(jobKey.getName());
                        } catch (SchedulerException e) {
                            log.error(e.getMessage(), e);
                            return null;
                        }
                    })
                    .filter(Objects::nonNull)
                    .collect(Collectors.toList());

        } catch (SchedulerException e) {
            log.error(e.getMessage(), e);
            return Collections.emptyList();
        }
        return null;
    }

    public Timerinfo getRunningTimer(String timerId) {
        try {
            final JobDetail jobDetail = scheduler.getJobDetail(new JobKey(timerId));
            return (Timerinfo) jobDetail.getJobDataMap().get(timerId);
        } catch (SchedulerException e) {
            log.error(e.getMessage(),e);
            return null;
        }
    }

    @PreDestroy
    public void preDestroy() {
        try {
            scheduler.shutdown();
        } catch (SchedulerException e) {
            log.error(e.getMessage(), e);
        }
    }

}
